//
//  CLRealPersonManager.h
//  CLRealPersonSDK
//
//  Created by ydong on 2021/6/5.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import <CLCore/CLCoreHttpResopnse.h>

/**
 @abstract    枚举
 
 @说明        CLCompletionHandler    对象的参数，用于表示获取token的状态
 
             CLCheckPass            活体检测通过
             CLCheckNotPass         活体检测不通过
             CLOperationTimeout     操作超时，用户未在规定时间内完成动作
             CLGetConfFaild         获取配置信息失败
             CLRecordFaild          视频录制失败
             CLOnlineCheckFaild     请求检测结果失败
             CLOnlineUploadFaild    上传图片失败
             CLSDKError             SDK内部发生错误
             CLRPVedioNoMate,       人脸动作与提示动作不吻合，请重试
             CLRPVedioNoFace,       没有检测到人脸，请重试
             CLRPVedioMoreFace,     检测到有多张人脸，请重试
             CLRPVideoIllumination, 检测到光线不足，请重试
             CLRPVedioAngle,        检测到人脸角度有偏差，请重试
             CLRPVedioBlue,         检测到人脸模糊，请重试
             CLRPVedioMask,         检测到人脸有遮挡，请重试
             CLRPVedioSmall         检测到人脸过于靠近屏幕，请重试
             CLNetError             无网络链接
             CLRPNoMoney            账号余额不足
             CLRPVedioError         其他错误
 */
typedef NS_ENUM(NSUInteger, CLStatus) {
    CLCheckPass = 1,
    CLCheckNotPass,
    CLOperationTimeout,
    CLGetConfFaild,
    CLRecordFaild,
    CLOnlineCheckFaild,
    CLOnlineUploadFaild,
    CLSDKError,
    CLRPVedioNoMate,
    CLRPVedioNoFace,
    CLRPVedioMoreFace,
    CLRPVideoIllumination,
    CLRPVedioAngle,
    CLRPVedioBlue,
    CLRPVedioMask,
    CLRPVedioSmall,
    CLNetError,
    CLRPNoMoney,
    CLRPVedioError
};

typedef NS_ENUM(NSUInteger, CLSecurityLevel) {
    CLSecurityLevelLow = 0,
    CLSecurityLevelHigh
};

typedef NS_ENUM(NSUInteger, CLDistanceLevel) {
    CLDistanceLevelNearbyMore = -2,
    CLDistanceLevelNearby = -1,
    CLDistanceLevelDefault = 0,
    CLDistanceLevelFar = 1,
    CLDistanceLevelFarMore = 2
};


NS_ASSUME_NONNULL_BEGIN

typedef void(^CLCompletionHandler)(CLStatus status, NSDictionary * _Nullable params);

/** @说明        检测中相关回调相关格式：
                {
                    type = @"1";
                    result = @"1";
                }
                内容说明：
                @"type" @"result"                   参考提示：
                @"1"    @"1"      人脸符合检测要求  -请保持姿势不变
                @"1"    @"2"      无人脸          -请面向屏幕
                @"1"    @"3"      人脸不全         -请保持脸在检测框内
                @"1"    @"4"      人脸占比过小     -请靠近一点
                @"1"    @"5"      光线暗          -当前光线过暗，请调整
                @"1"    @"6"      人脸模糊        -当前光线模糊，请调整
                @"1"    @"7"      人脸有遮挡      -请露出全脸
                @"1"    @"8"      其他错误        -其他错误，请继续
                @"3"    @"1"      检测中
                @"4"    @"url"    本地录制活体视频url(供转发用)                
                        
                number为动作总的数量
                type为2 result为-1时index表示将要执行的动作内容
                        result不为-1时result表示将要执行的动作内容 index：表示动作的索引从0开始
                @"type" @"result" @"number" @“index”
                @"2"    @"-1"       @"3"     @"0"       准备开始3个动作（3为随机数字），（0表示）第一个为眨眼
                @"2"    @"0"        @"3"     @"1"       眨眼（index为动作索引）
                @"2"    @"4"        @"3"     @"1"       抬头（index为动作索引）
                @"2"    @"5"        @"3"     @"1"       低头（index为动作索引）
                @"2"    @"6"        @"3"     @"1"       张嘴（index为动作索引）
                @"2"    @"7"        @"3"     @"1"       摇头（index为动作索引）
 */
typedef void(^CLAcitionsHandler)(NSDictionary *params);


@interface CLRealPersonManager : NSObject

/// 设置SDK token，作为后续操作凭证
/// @param token                    从后台获取的token
+ (void)setToken:(NSString *)token;

/// 获取SDK版本号
- (NSString *)getSDKVersion;


/// 初始化活体检测对象
/// @param recordView               传入放置检测活体的recordView对象
- (instancetype)initWithRecordView:(UIView *)recordView;

/// 设置活体检测的超时时间（在开始活体检测之前设置）
/// @param timeout                  请传入10-120范围内的时间值，单位s
- (void)setTimeoutInterval:(NSTimeInterval)timeout;

/// 设置活体检测每个动作的时长（在开始活体检测之前设置）
/// @param duration                 每个动作的时长（1-5s）
- (void)setActionDuration:(CGFloat)duration;

/// 设置检测时脸离屏幕距离（在开始活体检测之前设置）
/// @param distanceLevel                 默认0（-1：稍近，0：正常，1：稍远）
- (void)setDistanceNumber:(CLDistanceLevel)distanceLevel;

/// 设置活体检测动作的个数（在开始活体检测之前设置，默认为1个动作）
/// @param number                   动作的个数（1-3）
/// @param securityLevel            安全级别：默认为低（移动动作），高至少为两个动作(会修正number设置为1的数量)
- (void)setActionNumber:(NSInteger)number securityLevel:(CLSecurityLevel)securityLevel;

/// 开始活体检测
/// @param actionsHandler           活体检测动作序列号的回调（非主线程）
/// @param completionHandler        活体检测结果的回调（非主线程），结果状态见CLStatus枚举类型
- (void)startLiveDetectWithActionsHandler:(CLAcitionsHandler)actionsHandler
                    completionHandler:(CLCompletionHandler)completionHandler;

/// 活体检测，视频转发
/// @param urlString                视频转发之后URL或者startLiveDetectWithActionsHandler：返回的url
/// @param isReTran                 表示是否转发，YES：转发urlString参数值为转发后的url NO：不转发urlString参数值为startLiveDetectWithActionsHandler：中返回的url
- (void)resultWithDetectUrl:(NSString *)urlString
                   isReTran:(BOOL)isReTran;

/// 停止活体检测
/// 调用时机：
///     1、在活体检测结果的回调里调用
///     2、未完成活体检测，需要中止时调用 (超时不要调用)
- (void)stopLiveDetect;

@end

NS_ASSUME_NONNULL_END
